"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NemgenService = void 0;
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const fs_1 = require("fs");
const Docker = require("dockerode");
// eslint-disable-next-line @typescript-eslint/no-var-requires
const MemoryStream = require('memorystream');
const BootstrapUtils_1 = require("./BootstrapUtils");
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class NemgenService {
    constructor(params) {
        this.params = params;
    }
    async run(networkIdentifier, symbolServerToolsImage) {
        const docker = new Docker({ socketPath: '/var/run/docker.sock' });
        await BootstrapUtils_1.BootstrapUtils.pullImage(docker, symbolServerToolsImage);
        const nemesisSeedFolder = `${this.params.target}/data/nemesis-data/seed/${networkIdentifier}/0000`;
        await BootstrapUtils_1.BootstrapUtils.mkdir(nemesisSeedFolder);
        await fs_1.promises.copyFile(`${this.params.root}/config/hashes.dat`, `${nemesisSeedFolder}/hashes.dat`);
        const cmd = [
            'bash',
            '-c',
            'cd /data && /usr/catapult/bin/catapult.tools.nemgen  -r /userconfig --nemesisProperties /nemesis/block-properties-file.properties',
        ];
        const dir = `${process.cwd()}/${this.params.target}`;
        const binds = [
            `${dir}/config/generated-addresses:/addresses`,
            `${dir}/config/peer-node-0:/userconfig`,
            `${dir}/config/nemesis:/nemesis`,
            `${dir}/data/nemesis-data:/data:rw`,
        ];
        const createOptions = { User: await BootstrapUtils_1.BootstrapUtils.getDockerUserGroup(), HostConfig: { Binds: binds } };
        const startOptions = {};
        const memStream = new MemoryStream();
        let stdout = '';
        memStream.on('data', (data) => {
            const string = data.toString();
            // process.stdout.write(string);
            stdout += string;
        });
        logger.info('Running nemgen...');
        await docker.run(symbolServerToolsImage, cmd, memStream, createOptions, startOptions);
        if (stdout.indexOf('<error> ') > -1) {
            logger.info(stdout);
            throw new Error('Nemgen failed. Check the logs!');
        }
        logger.info('Nemgen executed!!!!');
    }
}
exports.NemgenService = NemgenService;
